@@ -6,10 +6,11 @@ import gevent.greenlet
 
 from cthulhu.gevent_util import nosleep
 from cthulhu.log import log
-from calamari_common.types import OsdMap, Health, MonStatus, ServiceId, MON, OSD, MDS
+from calamari_common.types import OsdMap, Health, MonStatus, QuorumStatus, ServiceId, MON, OSD, MDS, INFO, severity_str, WARNING, \
+    RECOVERY, ERROR, SEVERITIES
 from cthulhu.manager import config
-from calamari_common.db.event import Event, ERROR, WARNING, RECOVERY, INFO, severity_str
 from cthulhu.util import now
+from distutils.util import strtobool
 
 
 # The tick handler is very cheap (no I/O) so we call
@@ -26,6 +27,28 @@ GRACE_PERIOD = 30
 CONTACT_THRESHOLD_FACTOR = int(config.get('cthulhu', 'server_timeout_factor'))  # multiple of contact period
 CLUSTER_CONTACT_THRESHOLD = int(config.get('cthulhu', 'cluster_contact_threshold'))  # in seconds
 
+MINION_CONFIG = str(config.get('cthulhu', 'salt_config_path')).replace('master', 'minion')
+EMIT_EVENTS_TO_SALT_EVENT_BUS = bool(strtobool(config.get('cthulhu', 'emit_events_to_salt_event_bus')))
+EVENT_TAG_PREFIX = str(config.get('cthulhu', 'event_tag_prefix'))
+
+
+if EMIT_EVENTS_TO_SALT_EVENT_BUS:
+    try:
+        # TODO move this to import
+        # from calamari_common import Caller
+        import salt.client
+    except ImportError as e:
+        EMIT_EVENTS_TO_SALT_EVENT_BUS = False
+        log.error("Could not import salt.client: %s. Events cannot be emitted to salt event bus", str(e))
+
+
+class Event(object):
+    def __init__(self, severity, message, **associations):
+        self.severity = severity
+        self.message = message
+        self.associations = associations
+        self.when = now()
+
 
 class Eventer(gevent.greenlet.Greenlet):
     """
@@ -45,6 +68,14 @@ class Eventer(gevent.greenlet.Greenlet):
         self._servers_complained = set()
         self._clusters_complained = set()
 
+        # Check the config to decide if events has to be pushed to salt event bus.
+        # If config is set initialize the salt caller object used to push events.
+        if EMIT_EVENTS_TO_SALT_EVENT_BUS:
+            log.debug("Events will be emitted to salt event bus")
+            __opts__ = salt.config.minion_config(MINION_CONFIG)
+            __opts__['file_client'] = 'local'
+            self.caller = salt.client.Caller(mopts=__opts__)
+
         self._events = []
 
     def stop(self):
@@ -52,7 +83,9 @@ class Eventer(gevent.greenlet.Greenlet):
         self._complete.set()
 
     def _run(self):
+        log.debug("Eventer running")
         self._emit(INFO, "Calamari server started")
+        self._emit_to_salt_bus(SEVERITIES[INFO], "Calamari server started", "ceph/calamari/started")
         self._flush()
 
         self._complete.wait(GRACE_PERIOD)
@@ -61,6 +94,29 @@ class Eventer(gevent.greenlet.Greenlet):
             self._complete.wait(TICK_SECONDS)
         log.debug("Eventer complete")
 
+    def _emit_to_salt_bus(self, severity, message, tag, **tags):
+        """
+        This function emits events to salt event bus, if the config
+        value "emit_events_to_salt_event_bus" is set to true.
+        """
+        log.debug("Eventer running _emit_salt")
+        if not EMIT_EVENTS_TO_SALT_EVENT_BUS:
+            return
+
+        log.debug("Eventer running _emit_salt")
+        res = {}
+        res["message"] = message
+        res["severity"] = severity
+        res["tags"] = tags
+        tag = EVENT_TAG_PREFIX + tag
+
+        log.debug("Eventer._emit_to_salt_bus: Tag:%s | Data: %s" % (str(tag), str(res)))
+
+        self.caller.sminion.functions['event.send'](
+            tag,
+            res
+        )
+
     def _emit(self, severity, message, **associations):
         """
         :param severity: One of the defined serverity values
@@ -68,15 +124,9 @@ class Eventer(gevent.greenlet.Greenlet):
         :param associations: Optional extra attributes to associate
                              the event with a particular cluster/server/service
         """
-        now_utc = now()
-        log.info("Eventer._emit: %s/%s/%s" % (now_utc, severity_str(severity), message))
+        log.info("Eventer._emit: %s/%s" % (severity_str(severity), message))
 
-        self._events.append(Event(
-            when=now_utc,
-            message=message,
-            severity=severity,
-            **associations
-        ))
+        self._events.append(Event(severity, message, **associations))
 
     def on_user_request_begin(self, request):
         self._emit(INFO, "Started: %s" % request.headline, **request.associations)
@@ -95,6 +145,13 @@ class Eventer(gevent.greenlet.Greenlet):
             self._manager.persister.save_events(self._events)
             self._events = []
 
+    def reset_event_sink(self):
+        if EMIT_EVENTS_TO_SALT_EVENT_BUS:
+            log.debug("resetting minion")
+            __opts__ = salt.config.minion_config(MINION_CONFIG)
+            __opts__['file_client'] = 'local'
+            self.caller = salt.client.Caller(mopts=__opts__)
+
     # TODO consume messages about ServiceState from ServerMonitor, so that
     # we can tell people about their services in the absence of up to date
     # cluster map information.
@@ -110,7 +167,7 @@ class Eventer(gevent.greenlet.Greenlet):
         }
         return "{count} {human_service}{pluralize}".format(
             count=service_count,
-            human_service=human_singular[service_type],
+            human_service=human_singular.get(service_type, service_type),
             pluralize="s" if service_count > 1 else ""
         )
 
@@ -141,6 +198,7 @@ class Eventer(gevent.greenlet.Greenlet):
                 for (service_type, count) in counts_by_type.items()])
 
         self._emit(INFO, msg, fqdn=server_state.fqdn, fsid=self._server_fsid(server_state))
+        self._emit_to_salt_bus(SEVERITIES[INFO], msg, "ceph/server/added", fqdn=server_state.fqdn, fsid=self._server_fsid(server_state))
 
     @nosleep
     def on_reboot(self, server_state, expected):
@@ -151,6 +209,13 @@ class Eventer(gevent.greenlet.Greenlet):
                          we told it to reboot).  False indicates spontaneity)
         """
         severity = INFO if expected else WARNING
+        self._emit_to_salt_bus(
+            SEVERITIES[severity],
+            "Server {fqdn} rebooted".format(fqdn=server_state.fqdn),
+            "ceph/server/reboot",
+            fqdn=server_state.fqdn,
+            fsid=self._server_fsid(server_state)
+        )
         self._emit(severity,
                    "Server {fqdn} rebooted".format(fqdn=server_state.fqdn),
                    fqdn=server_state.fqdn,
@@ -167,6 +232,11 @@ class Eventer(gevent.greenlet.Greenlet):
         else:
             msg = "Ceph uninstalled from {fqdn}".format(fqdn=server_state.fqdn)
 
+        self._emit_to_salt_bus(
+            SEVERITIES[INFO], msg, "ceph/server/package/changed",
+            fqdn=server_state.fqdn,
+            fsid=self._server_fsid(server_state)
+        )
         self._emit(INFO, msg,
                    fqdn=server_state.fqdn,
                    fsid=self._server_fsid(server_state))
@@ -198,12 +268,20 @@ class Eventer(gevent.greenlet.Greenlet):
             contact_threshold = CONTACT_THRESHOLD_FACTOR * self._manager.servers.get_contact_period(fqdn)
             if now_utc - server_state.last_contact > datetime.timedelta(seconds=contact_threshold):
                 if fqdn not in self._servers_complained:
+                    self._emit_to_salt_bus(
+                        SEVERITIES[WARNING], "Server {fqdn} is late reporting in, last report at {last}".format(
+                            fqdn=fqdn, last=server_state.last_contact),
+                        "ceph/server/lateReporting", fqdn=fqdn, fsid=fsid
+                    )
                     self._emit(WARNING, "Server {fqdn} is late reporting in, last report at {last}".format(
                         fqdn=fqdn, last=server_state.last_contact
                     ), fqdn=fqdn, fsid=fsid)
                     self._servers_complained.add(fqdn)
             else:
                 if fqdn in self._servers_complained:
+                    self._emit_to_salt_bus(SEVERITIES[RECOVERY], "Server {fqdn} regained contact".format(fqdn=fqdn),
+                                           "ceph/server/regainedContact",
+                                           fqdn=fqdn, fsid=fsid)
                     self._emit(RECOVERY, "Server {fqdn} regained contact".format(fqdn=fqdn),
                                fqdn=fqdn, fsid=fsid)
                     self._servers_complained.discard(fqdn)
@@ -215,10 +293,15 @@ class Eventer(gevent.greenlet.Greenlet):
                     self._clusters_complained.add(fsid)
                     self._emit(WARNING, "Cluster '{name}' is late reporting in".format(name=cluster_monitor.name),
                                fsid=fsid)
+                    self._emit_to_salt_bus(SEVERITIES[WARNING], "Cluster '{name}' is late reporting in".format(name=cluster_monitor.name),
+                                           "ceph/cluster/lateReporting",
+                                           fsid=fsid)
             else:
                 if fsid in self._clusters_complained:
                     self._emit(RECOVERY, "Cluster '{name}' regained contact".format(name=cluster_monitor.name),
                                fsid=fsid)
+                    self._emit_to_salt_bus(SEVERITIES[RECOVERY], "Cluster '{name}' regained contact".format(name=cluster_monitor.name),
+                                           "ceph/cluster/regainedContact", fsid=fsid)
                     self._clusters_complained.discard(fsid)
 
         self._flush()
@@ -249,7 +332,21 @@ class Eventer(gevent.greenlet.Greenlet):
         deleted_osds = old_osd_ids - new_osd_ids
         created_osds = new_osd_ids - old_osd_ids
 
-        def osd_event(severity, msg, osd_id):
+        def osd_event(severity, msg, osd_id, osd_uuid):
+            self._emit_to_salt_bus(
+                SEVERITIES[severity],
+                msg.format(
+                    name=self._manager.clusters[fsid].name,
+                    id=osd_id,
+                    on_server=self._get_on_server(fsid, 'osd', osd_id)
+                ), "ceph/osd/propertyChanged",
+                fsid=fsid,
+                fqdn=self._get_fqdn(fsid, 'osd', osd_id),
+                service_type='osd',
+                service_id=str(osd_id),
+                osd_uuid=osd_uuid
+            )
+
             self._emit(
                 severity,
                 msg.format(
@@ -264,11 +361,11 @@ class Eventer(gevent.greenlet.Greenlet):
 
         # Generate events for removed OSDs
         for osd_id in deleted_osds:
-            osd_event(INFO, "OSD {name}.{id}{on_server} removed from the cluster map", osd_id)
+            osd_event(INFO, "OSD {name}.{id}{on_server} removed from the cluster map", osd_id, old.osds_by_id[osd_id]['uuid'])
 
         # Generate events for added OSDs
         for osd_id in created_osds:
-            osd_event(INFO, "OSD {name}.{id}{on_server} added to the cluster map", osd_id)
+            osd_event(INFO, "OSD {name}.{id}{on_server} added to the cluster map", osd_id, new.osds_by_id[osd_id]['uuid'])
 
         # Generate events for changed OSDs
         for osd_id in old_osd_ids & new_osd_ids:
@@ -276,9 +373,9 @@ class Eventer(gevent.greenlet.Greenlet):
             new_osd = new.osds_by_id[osd_id]
             if old_osd['up'] != new_osd['up']:
                 if bool(new_osd['up']):
-                    osd_event(RECOVERY, "OSD {name}.{id} came up{on_server}", osd_id)
+                    osd_event(RECOVERY, "OSD {name}.{id} came up{on_server}", osd_id, new.osds_by_id[osd_id]['uuid'])
                 else:
-                    osd_event(WARNING, "OSD {name}.{id} went down{on_server}", osd_id)
+                    osd_event(WARNING, "OSD {name}.{id} went down{on_server}", osd_id, new.osds_by_id[osd_id]['uuid'])
 
                     # TODO: aggregate OSD notifications by server so that we can say things
                     # like "all the OSDs on server X went down" or "2/3 OSDs on server X went down"
@@ -290,12 +387,63 @@ class Eventer(gevent.greenlet.Greenlet):
                     # with the server, and we haven't already reported the server laggy,
                     # to indicate that our best guess here is that the server itself is down.
 
+    def _on_pool_status(self, fsid, new, old):
+        old_pool_ids = set([o['pool'] for o in old.data['pools']])
+        new_pool_ids = set([o['pool'] for o in new.data['pools']])
+        deleted_pools = old_pool_ids - new_pool_ids
+        created_pools = new_pool_ids - old_pool_ids
+
+        def pool_event(severity, msg, pool_id, tag):
+            self._emit_to_salt_bus(
+                SEVERITIES[severity],
+                msg.format(
+                    name=self._manager.clusters[fsid].name,
+                    id=pool_id,
+                    on_server=self._get_on_server(fsid, 'pool', pool_id)
+                ), tag,
+                fsid=fsid,
+                fqdn=self._get_fqdn(fsid, 'pool', pool_id),
+                service_type='pool',
+                service_id=str(pool_id),
+            )
+
+            self._emit(
+                severity,
+                msg.format(
+                    name=self._manager.clusters[fsid].name,
+                    id=pool_id,
+                    on_server=self._get_on_server(fsid, 'pool', pool_id)
+                ),
+                fsid=fsid,
+                fqdn=self._get_fqdn(fsid, 'pool', pool_id),
+                service_type='pool',
+                service_id=str(pool_id))
+
+        # Generate events for removed pools
+        for pool_id in deleted_pools:
+            pool_event(INFO, "pool {name}.{id}{on_server} removed from cluster {name}", pool_id, 'ceph/pool/deleted')
+
+        # Generate events for added pools
+        for pool_id in created_pools:
+            pool_event(INFO, "pool {name}.{id}{on_server} added to cluster {name}", pool_id, 'ceph/pool/added')
+
     def _on_mon_status(self, fsid, new, old):
         old_quorum = set(old.data['quorum'])
         new_quorum = set(new.data['quorum'])
 
         def _mon_event(severity, msg, mon_rank):
             name = new.mons_by_rank[mon_rank]['name']
+            self._emit_to_salt_bus(
+                SEVERITIES[severity],
+                msg.format(
+                    cluster_name=self._manager.clusters[fsid].name,
+                    mon_name=name,
+                    on_server=self._get_on_server(fsid, 'mon', name)
+                ), "ceph/mon/propertyChanged",
+                fsid=fsid,
+                fqdn=self._get_fqdn(fsid, 'mon', name)
+            )
+
             self._emit(severity,
                        msg.format(
                            cluster_name=self._manager.clusters[fsid].name,
@@ -310,6 +458,33 @@ class Eventer(gevent.greenlet.Greenlet):
         for rank in old_quorum - new_quorum:
             _mon_event(WARNING, "Mon '{cluster_name}.{mon_name}' left quorum{on_server}", rank)
 
+    def _on_quorum_status(self, fsid, new, old):
+        old_leader_name = str(old.data['quorum_leader_name'])
+        new_leader_name = str(new.data['quorum_leader_name'])
+
+        def _leader_event(severity, msg, name):
+            self._emit_to_salt_bus(
+                SEVERITIES[severity],
+                msg.format(
+                    cluster_name=self._manager.clusters[fsid].name,
+                    mon_name=name,
+                    on_server=self._get_on_server(fsid, 'mon', name)
+                ), "ceph/mon/leaderChanged",
+                fsid=fsid,
+                fqdn=self._get_fqdn(fsid, 'mon', name)
+            )
+
+            self._emit(severity,
+                       msg.format(
+                           cluster_name=self._manager.clusters[fsid].name,
+                           mon_name=name,
+                           on_server=self._get_on_server(fsid, 'mon', name)),
+                       fsid=fsid,
+                       fqdn=self._get_fqdn(fsid, 'mon', name))
+
+        if old_leader_name != new_leader_name:
+            _leader_event(INFO, "Mon '{cluster_name}.{mon_name}' now quorum leader {on_server}", new_leader_name)
+
     def _on_health(self, fsid, new, old):
         # Generate notifications for transitions between HEALTH_OK, HEALTH_WARN, HEALTH_ERR
         old_status = old.data['overall_status']
@@ -337,6 +512,7 @@ class Eventer(gevent.greenlet.Greenlet):
                 # which will soon be stale
                 pass
                 # msg += " (%s)" % (new.data['summary'][0]['summary'])
+            self._emit_to_salt_bus(SEVERITIES[event_sev], msg, "ceph/cluster/health/changed", fsid=fsid)
             self._emit(event_sev, msg, fsid=fsid)
 
     @nosleep
@@ -357,11 +533,14 @@ class Eventer(gevent.greenlet.Greenlet):
             return
 
         if sync_type == OsdMap:
+            self._on_pool_status(fsid, new, old)
             self._on_osd_map(fsid, new, old)
         elif sync_type == Health:
             self._on_health(fsid, new, old)
         elif sync_type == MonStatus:
             self._on_mon_status(fsid, new, old)
+        elif sync_type == QuorumStatus:
+            self._on_quorum_status(fsid, new, old)
 
         self._flush()
 
